/*****************************************************************************
 This file is part of the Changedfiles system. 
 $Id: utils.c,v 1.8 2003/02/03 04:43:01 rocky Exp $

 This product includes software developed by Konrad Rieck.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.

 This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 USA
******************************************************************************/
/*
  This file contains some miscellaneous utility routines, and some
  routines that are in libc but can't be used in kernel space.
*/

#define _KERNEL

#include "headers.h"
#include "libc.h"
#include "comm.h"

#include <limits.h>
#include <unistd.h>
/* This is to prevent a conflicting definition of ffs in string.h */
#define _POSIX_C_SOURCE 1
#include <string.h>

/* Should match chf_syscall_t in changedfiles.h */
const char *op2str[] = {
  "OPEN",  "OPEN64", "RMDIR", "MKDIR",  "SYMLINK", "LINK", "UNLINK", "CREATE", 
  "CLOSE", "RENAME", "CHMOD", "PUTMSG", "PAPCOMM", NULL
};

#define SEPERATOR "]]]----->>"

static int
chf_expand_filename( const char *name, char *fullname ) 
{
  fullname[0]='\0';

  /* basename file names, fill path if neccessary */ 
  if ( name[0] == '/' ) { 
    strncpy(fullname, name, PATH_MAX); 
    return 1;
  }

#ifdef FIXED
  char *cwd;
  /*cwd=getcwd(fullname, PATH_MAX);*/
  if ( NULL == cwd ) { 
    cmn_err(CE_WARN, "changedfiles: getcwd returned an error"); 
    return 0; 
  }
  
  if ( ( chf_strlen( fullname ) + chf_strlen( name )) >= PATH_MAX ){ 
    long int len= ( chf_strlen( fullname ) + chf_strlen( name )); 
    cmn_err(CE_WARN, 
	    "changedfiles: Full filename too long! %d bytes.  Max: %d", 
	    (int)len, PATH_MAX); 
    return 0; 
  }
  
  chf_strncat(fullname, "/", PATH_MAX); 
  chf_strncat(fullname, name, PATH_MAX); 
#else
  strncpy(fullname, name, PATH_MAX); 
#endif
  return 1;
}


/* 
   This is the routine logs what's happening so that user-space
   daemon can take action.
*/
int
chf_log_filename( char * oldname, char * newname, 
		  const chf_syscall_op operation ) 
{ 
  char msg[PATH_MAX * 2 + 80]; 
  char fulloldname[PATH_MAX];
  char fullnewname[PATH_MAX];
  char int_str[30];
  unsigned int len;
  
  int_str[0]='\0';
  msg[0]='\0';
  
  /* basename file names, fill path if neccessary */ 
  if (!chf_expand_filename(oldname, fulloldname))
    return 0;

  len = chf_uitostr(int_str, ddi_get_pid());
  chf_strncat(msg, int_str, len);
  chf_strncat(msg, " ", PATH_MAX);
  chf_strncat(msg, op2str[operation], PATH_MAX);
  chf_strncat(msg, " ", PATH_MAX);
  chf_strncat(msg, fulloldname, PATH_MAX);
  if (newname != NULL) {
    if (!chf_expand_filename(newname, fullnewname))
      return 0;
    chf_strncat(msg, SEPERATOR, PATH_MAX);
    chf_strncat(msg, fullnewname, PATH_MAX); 
  }

  if ( msg[0] != '\0' ) { 
      chf_put_to_queue(msg);
  }
  
  return 1;
}
